Program IPX_Chat;

(*****************************************)
(*    Written by Alexei A. Frounze (c)   *)
(*        on 27 of September 1999        *)
(*                                       *)
(*     Target:  Real/Protected mode      *)
(*  Compiler: Borland/Turbo Pascal 7.0   *)
(*                                       *)
(* E-mail: alexfru@chat.ru               *)
(* Homepage: http://www.chat.ru/~alexfru *)
(*****************************************)

Uses IPX, CRT;

                         (* GENERAL CONSTANTS *)

Const
  DefaultSocket = $5000;                        { Default socket for the program }
  DoomSocket    = $869C;                        { Official Doom socket :) }
  ChatID        = 'IPX_CHAT_MSG';               { Chat message ID }
  MaxMessages   = 20;                           { Max messages at the screen }

                 (* 512-BYTES DATA BLOCK TYPE DEFINITION *)

Type
  TData = Record
    Typ : Byte;
    Msg : String;
    Res : Array[0..254] of Byte
  End;

                               (* VARIABLES *)

Var
  { Chat socket number }
  ChatSocket : Word;                            { Not reversed!!! }

  Msgs : Array[0..MaxMessages-1] of String;     { Received messages }
  MsgCnt,                                       { Received messages number }

  I : Integer;
  Res : Byte;
  Key : Char;                                   { Character from keyboard }
  Msg : String;                                 { Own message }
  GotMsg : Boolean;                             { Flag of available msg(s) }
  DataBlock : Array[0..MaxUsers-1] of TData;    { 512-bytes data blocks }
  FromNode : Array[0..MaxUsers-1] of Array [0..5] of Byte; { Senders that sent mesg(s) }

                    (* STRING HANDLING FUNCTIONS *)

Function HexB (B : Byte) : String;
Const
  HC : Array [0..15] of Char = '0123456789ABCDEF';
Begin
  HexB := HC[B shr 4] + HC[B and 15]
End;

Function ToHex (Var Buf; Count : Integer) : String;
Var
  X : Array[0..255] of Byte Absolute Buf;
  I : Byte;
  S : String;
Begin
  S := '';
  For I := 0 to Count-1 do
    S := S + HexB(X[I]);
  ToHex := S
End;

                      (* SENDING/RECEIVING ROUTINES *)

Procedure Send_Message (Message : String);
Var
  Data : TData;
  ToNode : Array [0..5] of Byte;
Begin
  Data.Typ := 1;
  Data.Msg := ChatID+Message;
  FillChar (ToNode, 6, $FF);
  IPX_Send_Data (Data, ToNode)
End;
                  (* MESSAGES/SCREEN HANDLING ROUTINES *)

Procedure Clear_Messages;
Var
  I : Integer;
Begin
  For I := 0 to MaxMessages-1 do
    Msgs[I] := '';
  MsgCnt := 0
End;

Procedure Add_Message (Msg : String);
Var I : Integer;
Begin
  If MsgCnt < MaxMessages then Begin
    Msgs[MsgCnt] := Msg;
    Inc (MsgCnt)
  End
  Else Begin
    For I := 0 to MaxMessages-2 do
      Msgs[I] := Msgs[I+1];
    Msgs[MaxMessages-1] := Msg
  End
End;

Procedure Display_Messages;
Var I : Integer;
Begin
  For I := 0 to MaxMessages-1 do Begin
    GotoXY (1, 2+I); Write (Msgs[I]); ClrEol
  End
End;

Procedure Display_Own_Message;
Begin
  GotoXY (1, MaxMessages+3);
  ClrEol;
  Write ('>',Msg)
End;

                             (* MAIN PROGRAN *)

Label LEnd;

Begin
  WriteLn ('IPX Chat v1.0 by Alexei A. Frounze (c) 1999');
  If ParamCount <> 0 then Begin
    If ParamStr(1) = '/?' then Begin
      WriteLn ('Usage:  IPX_CHAT.EXE  [SOCKET_NUMBER]   <');
      WriteLn ('Default socket number is 20480 (5000h).');
      WriteLn ('You can also ask IPX-driver for available socket number, ');
      WriteLn ('sipmly type: IPX_CHAT.EXE 0 <');
      Goto LEnd
    End;
    Val (ParamStr(1), ChatSocket, I);
    If I <> 0 then Begin
      WriteLn ('You didn''t enter a number.');
      Goto LEnd
    End
  End
  Else ChatSocket := DefaultSocket;

  If not Is_IPX_Installed then Begin
    WriteLn ('IPX protocol is not installed.');
    Goto LEnd
  End;

  WriteLn (#13#10'IPX protocol is installed.');

  IPX_Get_Full_Address (_Addr);
  WriteLn (#13#10'Local area network address is: ', ToHex(_Addr.NetAddress,4));
  WriteLn ('Node is: ', ToHex(_Addr.Node,6));

  WriteLn (#13#10'Trying to open ', ChatSocket, ' socket...');
  { gives free socket ($4000...$4FFF) by socket=0 request }
  Res := IPX_Open_Socket (False, ChatSocket);             { Openning socket }
  If Res <> 0 then Begin
    WriteLn ('Couldn''t open desired socket.');
    Goto LEnd
  End;
  WriteLn ('Successfully. Socket ', ChatSocket, ' is open.');
  _Addr.Socket := ((ChatSocket and $FF) shl 8) + (ChatSocket shr 8); { Reversed!!! }
  WriteLn ('Be sure other users enter the chat using this socket.');

  WriteLn (#13#10'You should at least see "',ToHex(_Addr.Node,6),
                 '> On-line" message in the chat window.');

  Write (#13#10'Hit any key to enter the chat...');
  While not KeyPressed do; While KeyPressed do ReadKey;

  ClrScr;
  For I := 1 to 80 do Begin
    GotoXY (I, 1); Write ('-');
    GotoXY (I, MaxMessages+2); Write ('-');
  End;
  Msg := '';
  Display_Own_Message;

  IPX_Prepare_For_Receiving (0, MaxUsers-1);    { Listen for IPX packets }

  Send_Message ('On-line');             { We should recive this msg too! }

                                (* MAIN LOOP *)

  Repeat
    IPX_Relinquish_Control;             { Let IPX driver do some work }
    GotMsg := False;
    For I := 0 to MaxUsers-1 do
      If IPX_Receive_Data (I, DataBlock[I], FromNode[I]) then Begin
        If Pos(ChatID,DataBlock[I].Msg)=1 then Begin         { Got new msg }
          Delete (DataBlock[I].Msg, 1, Length(ChatID));
          Add_Message (ToHex(FromNode[I],6)+'> '+DataBlock[I].Msg);
          GotMsg := True
        End
      End;
    If GotMsg then Begin
      Display_Messages;                                 { Displaying it }
      Display_Own_Message                               { Your input string }
    End;

    If KeyPressed then Key := ReadKey
    Else Key := #$FF;
    Case Key of
      #0  : Begin ReadKey; Key := #$FF End;
      #27 : Break;
      #13 : Begin If Msg <> '' then Send_Message (Msg); Msg := '' End;
      #8  : If Msg <> '' then Dec(Msg[0]);
      #32..#254 : If Length(Msg) < 66 then Msg := Msg + Key
    End;
    If Key in [#8,#13,#32..#254] then
      Display_Own_Message
  Until False;

  Send_Message ('Off-line');                            { Say "goodbye" }
  IPX_Relinquish_Control;

  IPX_Close_Socket (ChatSocket);                        { Closing socket }
  ClrScr;
LEnd:
  FreeRMBufs;                                           { Freeing buffers }
End.
